/////////////////////////////////////////////////////////////////////////
// IR_IN project

#include "msp430g2553.h"
#include "stdarg.h"
#include "printf.h"

#ifndef TIMER0_A1_VECTOR
#define TIMER0_A1_VECTOR TIMERA1_VECTOR
#define TIMER0_A0_VECTOR TIMERA0_VECTOR
#endif

#define LED_RED 			BIT0
#define LED_GREEN 			BIT6
#define PORT_IN 			BIT4

#define RECEIVER_ON			1
#define RECEIVER_OFF		2
#define RECEIVER_ON_MUTE	3

#define MUTE_INTERVAL		1000		//       
										//       

#define VALUES_NUMBER		3

unsigned int i;
unsigned long circle = 0;
unsigned long circle_prev = 0;
unsigned int timervalue = 0;		//     
unsigned int timervalue_prev = 0;	//     
unsigned int value = 0;
unsigned int pause = 0;
unsigned int waitforvalue = 0;
unsigned int values[VALUES_NUMBER];
unsigned int values_index;
unsigned int true_value;
unsigned int value_is_true;
unsigned int timer1_value;

unsigned int value_off;

unsigned int zero_value = 677;		//   

unsigned int receiver_state = RECEIVER_ON;

void ClockSetup(void);
void PortsSetup(void);
void UARTSetup();
void TimerSetup(void);

void AddValue(unsigned int);
unsigned int CheckValues(void);
void ReceiverOff(void);

/*
 * main.c
 */
int main(void) {
    WDTCTL = WDTPW | WDTHOLD;	// Stop watchdog timer
	
    PortsSetup();
    ClockSetup();
    UARTSetup();
    TimerSetup();

    true_value = zero_value;

    printf("\r\n============= Start ==============\r\n");


    while(1)
    {
    	_bis_SR_register(LPM1_bits + GIE);
    }
}

void PortsSetup(void)
{
	P1DIR |= LED_RED + LED_GREEN + BIT5;
	P1OUT &= ~(LED_RED + LED_GREEN + BIT5);

	P2DIR |= BIT0;					// P2.0 -   
	P2OUT |= BIT0;


	P1SEL |= LED_GREEN + BIT5;		//   

	P1DIR &= ~PORT_IN;				//  - 
    P1REN &= ~PORT_IN; 				//  
	P1IES &= ~PORT_IN;				//     0  1
	P1IFG &= ~PORT_IN;				//  
	P1IE |= PORT_IN;				//  
}

void ClockSetup(void)
{
	BCSCTL1 = CALBC1_1MHZ; 	//    Basic Clock System.
	DCOCTL = CALDCO_1MHZ; 	//  
	                        //  Digital Controlled Oscillator.
	BCSCTL3 |= LFXT1S_0 + XCAP_0;             // LFXT1 = XT1
	while(IFG1 & OFIFG)						  // wait for OSCFault to clear
	{
	  IFG1 &= ~OFIFG;
	  _delay_cycles(10);
	}
	BCSCTL2 |= SELM_0;      // MCLK = DCO, SMCLK = DCO
}

void TimerSetup(void)
{
	//  1 - 
	TA1CCR0 = 32767;				//  - 1   
	TA1CTL = TASSEL_1 + MC_1;		// ACLK - 32768   
	TA1CCTL0 = CCIE;
	//TA1CCTL1 |= CCIE;

	//  0 -  
	CCR0 = 1023;				//   - 1 
	CCR1 = 511;
	TACTL = TASSEL_2 + MC_1;	//    SMCLK   1 
	TACCTL1 = OUTMOD_6;
	TACCTL0 = OUTMOD_4;
	//CCTL0 = CCIE;
}

void UARTSetup()
{
	P1SEL |= BIT1 + BIT2; 	//   P1.1  RXD  UART, P1.2  TXD
	P1SEL2 |= BIT1 + BIT2;	//

	UCA0CTL1 |= UCSWRST; 	//       UART  
	                       	//     
	                       	//  (   UART).
	UCA0CTL1 |= UCSSEL_2;   //  UART   
	                        // SMCLK (Sub-main clock),   1MHZ.
	UCA0BR0 = 0x68; 		//    SMCLK (1000000 / 9600).
	UCA0BR1 = 0x00;
	UCA0MCTL = 0x02; 		//   .
	                  	  	//    .
	UCA0CTL1 &= ~UCSWRST; 	//  UART .

	IE2 |= UCA0RXIE;
}


void CalculateValue(void)
{
	if (waitforvalue)
	{
		if (circle == circle_prev && timervalue - timervalue_prev < 4096)
		{
			value = (timervalue - timervalue_prev);
			AddValue(value);
			value_is_true = CheckValues();
			waitforvalue = 0;
		}
		else if (circle == circle_prev + 1 && timervalue + 32768 - timervalue_prev < 4096)
		{
			value = (timervalue + 32768 - timervalue_prev);
			AddValue(value);
			value_is_true = CheckValues();
			waitforvalue = 0;
		}
	}
	else
	{
		if (circle == circle_prev)
		{
			pause = timervalue - timervalue_prev;
		}
		else if (circle == circle_prev + 1)
		{
			pause = timervalue + 32768 - timervalue_prev;
		}

		if (pause > 16384)
		{
			waitforvalue = 1;
		}
	}

	timervalue_prev = timervalue;
	circle_prev = circle;
}

void AddValue(unsigned int val)
{
	values[values_index++] = val;
	if (values_index == VALUES_NUMBER)
	{
		values_index = 0;
	}
}

unsigned int CheckValues(void)
{
	for (i = 1; i < VALUES_NUMBER; i++)
	{
		if (values[0]/4 != values[i]/4)
		{
			return 0;
		}
	}
	true_value = values[0] / 4;
	ReceiverOff();
	return 1;
}

void UpdatePWM(void)
{
	CCR1 = (true_value - zero_value)*4 + 511;
}

void ReceiverOff(void)
{
	value_off = 32768 - values[0] - MUTE_INTERVAL - 100;

	if (timervalue + value_off < 32768)
	{
		TA1CCR1 = timervalue + value_off;
	}
	else
	{
		TA1CCR1 = timervalue + value_off - 32768;
	}

	//   ,    TA1CCR1,  
	receiver_state = RECEIVER_OFF;
	P2OUT = 0;
	TA1CCTL1 &= ~CCIFG;
	TA1CCTL1 |= CCIE;
}


//////////////////////////////////////////////////////////////////////////////////////////
//		 
//////////////////////////////////////////////////////////////////////////////////////////

// P1 interrupt
#pragma vector = PORT1_VECTOR
__interrupt void P1_ISR(void)
{
    switch(P1IFG & PORT_IN)
    {
        case PORT_IN:
            P1IFG &= ~PORT_IN;    //     PORT_IN

            if (receiver_state == RECEIVER_ON)	//   RECEIVER_ON  
            {
            	timervalue = TA1R;
            	P1OUT ^= LED_RED;
            	CalculateValue();
            }
            return;
        default:
            return;
    }
} // P1_ISR

// Timer 1 interrupt - 1   
#pragma vector=TIMER1_A0_VECTOR
__interrupt void Timer_A1 (void)
{
	circle++;
	printf("%l:\t%u\t%u\t%u\t%u\t%u\r\n", circle, true_value);
	UpdatePWM();
}

// Timer 1 interrupt -  CCR1 ->  
#pragma vector=TIMER1_A1_VECTOR
__interrupt void Timer_A (void)
{
	TA1CCTL1 &= ~CCIFG;
	if (receiver_state == RECEIVER_OFF)		//  ,    RECEIVER_ON_MUTE
	{
		P2OUT = BIT0;
		receiver_state = RECEIVER_ON_MUTE;

		if (TA1CCR1 + MUTE_INTERVAL < 32768)	//    MUTE_INTERVAL
		{
			TA1CCR1 += MUTE_INTERVAL;
		}
		else
		{
			TA1CCR1 += MUTE_INTERVAL - 32768;
		}
	}
	else if (receiver_state == RECEIVER_ON_MUTE)	//     RECEIVER_OFF
	{
		TA1CCTL1 &= ~CCIE;
		receiver_state = RECEIVER_ON;
	}
}


