/////////////////////////////////////////////////////////////////////////
// AM_OUT project

#include "msp430g2452.h"

#ifndef TIMER0_A1_VECTOR
#define TIMER0_A1_VECTOR TIMERA1_VECTOR
#define TIMER0_A0_VECTOR TIMERA0_VECTOR
#endif

#define LED_RED 			BIT0

#define PERIODS_NUMBER		2

#define SERIES_NUMBER 5							//     



unsigned int i;

unsigned int periodindex = 0;

unsigned int periods[PERIODS_NUMBER] = {5000, 32767 - 5000};

unsigned int valueT;							// Temperature
unsigned int seriesBuffer[SERIES_NUMBER];		//  

unsigned int channel;							// Sampling channel



void PortsSetup(void);
void ClockSetup(void);
void TimerSetup(void);
void StartSampling(void);

/*
 * main.c
 */
int main(void) {
    WDTCTL = WDTPW | WDTHOLD;	// Stop watchdog timer
	
    PortsSetup();
    ClockSetup();

	StartSampling();

    TimerSetup();

    while(1)
    {
    	_bis_SR_register(LPM3_bits + GIE);
    }
}

void PortsSetup(void)
{
	P1DIR |= LED_RED;
	P1OUT &= ~LED_RED;
}

void ClockSetup(void)
{
	BCSCTL1 = CALBC1_1MHZ; 		//    Basic Clock System.
	DCOCTL = CALDCO_1MHZ; 		//  
	                        	//  Digital Controlled Oscillator.
	BCSCTL3 |= LFXT1S_0;        // LFXT1 = XT1
	while(IFG1 & OFIFG)			// wait for OSCFault to clear
	{
		IFG1 &= ~OFIFG;
		_delay_cycles(10);
	}
	BCSCTL2 |= SELM_0;      	// MCLK = DCO, SMCLK = DCO
}

void TimerSetup(void)
{
	CCR0 = periods[periodindex++];
	TACTL = TASSEL_1 + MC_1;		// ACLK - 32768   
	CCTL0 = CCIE;
}

void TimerUpdate(void)
{
	CCR0 = periods[periodindex];
	periodindex++;
	if (periodindex == 1)
	{
		StartSampling();
	}
	else if (periodindex == PERIODS_NUMBER)
	{
		periodindex = 0;
	}
}

void StartSampling(void)
{
	ADC10CTL1 = ADC10SSEL_1 + CONSEQ_2;			// ACLC + Repeat single channel

	ADC10CTL1 |= INCH_10;					// Temp Sensor

	ADC10CTL0 = SREF_1 + ADC10SHT_0 + REFON + ADC10ON + ADC10IE + MSC;

	_delay_cycles(10);                           // Wait for ADC Ref to settle
	ADC10CTL0 &= ~ENC;
	while (ADC10CTL1 & BUSY);
	ADC10DTC1 = SERIES_NUMBER;
	ADC10SA = (unsigned int)&seriesBuffer;
	ADC10CTL0 |= ADC10SC + ENC;                 // Sampling and conversion start
}

unsigned int GetAverageFromSeries(void)
{
	unsigned int sum = 0;
	for (i = 0; i < SERIES_NUMBER; i++)
	{
		sum += seriesBuffer[i];
	}
	return sum/SERIES_NUMBER + ((sum % SERIES_NUMBER)*2 >= SERIES_NUMBER);
}

void ProcessResult(void)
{
	if (valueT > 0)
	{
		periods[0] = valueT * 4;
	}
	else
	{
		periods[0] = 4000;
	}
	periods[1] = 32767 - periods[0];
}

//////////////////////////////////////////////////////////////////////////////////////////
//		 
//////////////////////////////////////////////////////////////////////////////////////////


// Timer 1 interrupt -  ,   periods[]
#pragma vector=TIMER0_A0_VECTOR
__interrupt void Timer_A (void)
{
	P1OUT |= LED_RED;						//  - 2  
	P1OUT &= ~LED_RED;
	TimerUpdate();
}

// ADC10 interrupt
#pragma vector=ADC10_VECTOR
__interrupt void ADC10 (void)
{
	ADC10CTL0 &= ~ADC10IFG; 			    // clear interrupt flag
	ADC10CTL0 &= ~ENC;				        // Disable ADC conversion
	ADC10CTL0 &= ~ADC10ON;        			// ADC10 off
	ADC10CTL0 &= ~REFON;

	valueT = GetAverageFromSeries();	// T, then A
	ProcessResult();
}

